# Cog Stuff
import logging

# OPCALENDAR
import operator
from datetime import datetime
from itertools import chain

# AA Contexts
from allianceauth.services.modules.discord.models import DiscordUser
from app_utils.urls import static_file_absolute_url
from asgiref.sync import sync_to_async
from discord.colour import Color
from discord.commands import Option
from discord.embeds import Embed
from discord.ext import commands
from django.conf import settings
from django.db.models import F, Q

from opcalendar.app_settings import (
    OPCALENDAR_DISCORD_OPS_DISPLAY_EXTERNAL,
    get_site_url,
)
from opcalendar.models import Event, EventHost, IngameEvents

logger = logging.getLogger(__name__)


class Ops(commands.Cog):
    """
    Return upcoming events as a discord DM
    """

    def __init__(self, bot):
        self.bot = bot

    hosts = EventHost.objects.all().values_list("community", flat=True)

    @commands.command(pass_context=True)
    async def ops(self, ctx):
        """
        Sends a direct message about the upcoming events visible for the user
        """
        await ctx.trigger_typing()

        # Get authod ID
        author_id = ctx.message.author.id

        user_argument = ctx.message.content[5:]

        embed = await _get_events(author_id, user_argument)

        await ctx.author.send(embed=embed)

        confirm = _confirm_ops_in_channel()

        return await ctx.reply(embed=confirm)

    @commands.slash_command(name="ops", guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def ops_slash(
        self,
        ctx,
        host: Option(
            str,
            choices=hosts,
            required=False,
        ),
    ):
        """
        Get upcoming events as DM. Can be filtered by hosts (optional)
        """

        # Get attributes
        author_id = ctx.author.id

        user_argument = host

        embed = await _get_events(author_id, user_argument)

        await ctx.author.send(embed=embed)

        confirm = _confirm_ops_in_channel()

        return await ctx.respond(embed=confirm)


def _hosts(hosts):
    hosts = [x.community for x in hosts]

    if hosts:
        return ", ".join(hosts)
    else:
        return None


@sync_to_async
def _get_events(author, user_argument):
    url = get_site_url()

    today = datetime.today()

    if not user_argument:
        host = "all hosts"

    else:
        host = user_argument

    # Get user if discord service is active
    try:
        discord_user = DiscordUser.objects.get(uid=author)

        user = discord_user.user

        discord_active = True

    except Exception:
        logger.error("Discord service is not active for user")

        embed = Embed(title="Command failed")
        embed.colour = Color.red()
        embed.set_thumbnail(url=static_file_absolute_url("opcalendar/terminate.png"))
        embed.description = "Activate the [discord service]({}/services) to access this command.".format(
            url
        )
        discord_active = False

        return embed

    if discord_active:
        # Get normal events
        # Filter by groups and states
        events = (
            Event.objects.filter(
                Q(event_visibility__restricted_to_group__in=user.groups.all())
                | Q(event_visibility__restricted_to_group__isnull=True),
            )
            .filter(
                Q(event_visibility__restricted_to_state=user.profile.state)
                | Q(event_visibility__restricted_to_state__isnull=True),
            )
            .filter(start_time__gte=today)
            .distinct()
        )
        if user_argument:
            events = events.filter(host__community=host)

        # Get ingame events
        # Filter by groups and states
        ingame_events = (
            IngameEvents.objects.annotate(
                start_time=F("event_start_date"),
                end_time=F("event_end_date"),
            )
            .filter(
                Q(owner__event_visibility__restricted_to_group__in=user.groups.all())
                | Q(owner__event_visibility__restricted_to_group__isnull=True),
            )
            .filter(
                Q(owner__event_visibility__restricted_to_state=user.profile.state)
                | Q(owner__event_visibility__restricted_to_state__isnull=True),
            )
            .filter(start_time__gte=today)
            .distinct()
        )

        hosts = EventHost.objects.all()

        if not OPCALENDAR_DISCORD_OPS_DISPLAY_EXTERNAL:
            hosts = hosts.filter(external=False)

        hosts = _hosts(hosts)

        if user_argument:
            ingame_events = ingame_events.filter(host__community=host)

        # Combine events, limit to 10 events
        all_events = sorted(
            chain(events, ingame_events),
            key=operator.attrgetter("start_time"),
        )[:10]

        embed = Embed(title="Scheduled Opcalendar Events")

        embed.set_thumbnail(url=static_file_absolute_url("opcalendar/calendar.png"))

        embed.colour = Color.blue()

        embed.description = "List view of the next 10 upcoming operations for {}. A calendar view is located in [here]({}/opcalendar).\n\nFiltering: To filter events for a specific host add the name after the command ie. `/ops my coalition`\n\nAvailable hosts: *{}*".format(
            host, url, hosts
        )

        # Format all events and ingame events
        for event in all_events:
            if isinstance(event, Event):
                embed.add_field(
                    name="Event: {0} {1}".format(
                        event.title, event.operation_type.ticker
                    ),
                    value="Host: {0}\nFC: {1}\nDoctrine: {2}\nLocation: {3}\nTime: {4}\n[Details]({5}/opcalendar/event/{6}/details/)\n".format(
                        event.host,
                        event.fc,
                        event.doctrine,
                        event.formup_system,
                        event.start_time,
                        url,
                        event.id,
                    ),
                    inline=False,
                )
            if isinstance(event, IngameEvents):
                embed.add_field(
                    name="Ingame Event: {0}".format(event.title),
                    value="Host: {0}\n Time:{1}\n[Details]({2}/opcalendar/ingame/event/{3}/details/)".format(
                        event.owner_name,
                        event.start_time,
                        url,
                        event.event_id,
                    ),
                    inline=False,
                )

        discord_active = False

        return embed


def _confirm_ops_in_channel():
    embed = Embed(title="Events sent as DM!")
    embed.colour = Color.green()

    return embed


def setup(bot):
    bot.add_cog(Ops(bot))



# Generated by Django 5.2.15 on 2026-06-10 10:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("opcalendar", "0027_eventmember_comment_eventmember_status_usersettings"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="general",
            options={
                "default_permissions": (),
                "managed": False,
                "permissions": (
                    (
                        "basic_access",
                        "Can access this app and see operations based on visibility rules",
                    ),
                    ("create_event", "Can create and edit own events"),
                    ("see_signups", "Can see all signups for event"),
                    ("manage_event", "Can delete and manage own events"),
                    ("manage_all_events", "Can edit and delete all events"),
                    (
                        "add_ingame_calendar_owner",
                        "Can add ingame calendar feeds for their corporation",
                    ),
                ),
            },
        ),
        migrations.AddField(
            model_name="event",
            name="important",
            field=models.BooleanField(
                default=False,
                help_text="Mark this operation as important. Important operations show a blinking red indicator on the calendar so members notice them at a glance.",
            ),
        ),
        migrations.AddField(
            model_name="event",
            name="notification_custom_date",
            field=models.DateTimeField(
                blank=True,
                help_text="Custom date and time to send the Discord ping. Only used when 'Custom date and time' is selected above.",
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="event",
            name="notification_timings",
            field=models.JSONField(
                blank=True,
                default=list,
                help_text="When to ping this event on Discord. Select any combination of now, a relative time before fleet start, or a custom date and time.",
            ),
        ),
        migrations.AddField(
            model_name="event",
            name="notifications_sent",
            field=models.JSONField(
                blank=True,
                default=list,
                editable=False,
                help_text="Tracks which notification timings have already been sent",
            ),
        ),
        migrations.AddField(
            model_name="event",
            name="repeat_group_id",
            field=models.UUIDField(
                blank=True,
                db_index=True,
                help_text="Links all events created as part of the same repeat chain",
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="event",
            name="repeat_interval",
            field=models.PositiveIntegerField(
                blank=True,
                default=1,
                help_text="Repeat every X of the selected interval (e.g. 2 = every 2 weeks)",
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="eventcategory",
            name="icon",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Optional FontAwesome icon classes to display on the calendar for this category, e.g. 'fas fa-rocket' or 'far fa-clock' or 'fab fa-discord'. Enter the full set of classes exactly as you would in HTML — any icon style available in your AllianceAuth installation's bundled FontAwesome version works. The icon will be tinted using the color set above.",
                max_length=100,
            ),
        ),
        migrations.AlterField(
            model_name="event",
            name="repeat_event",
            field=models.CharField(
                blank=True,
                choices=[
                    ("DD", "Daily"),
                    ("WE", "Weekly"),
                    ("BW", "Every 2 weeks"),
                    ("MM", "Monthly"),
                    ("YY", "Yearly"),
                ],
                default=False,
                help_text="Select if you want to repeat this event in the future",
                max_length=32,
                null=True,
            ),
        ),
        migrations.CreateModel(
            name="EventChannel",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        help_text="A label to help you tell channels apart in the admin.",
                        max_length=150,
                    ),
                ),
                (
                    "channel_id",
                    models.BigIntegerField(
                        help_text="The Discord channel ID events should be posted to.",
                        unique=True,
                    ),
                ),
                (
                    "summary_message_id",
                    models.BigIntegerField(
                        blank=True,
                        help_text="Internal: ID of the pinned message showing the sorted schedule overview.",
                        null=True,
                    ),
                ),
                (
                    "enabled",
                    models.BooleanField(
                        default=True,
                        help_text="Whether this channel should currently receive event posts.",
                    ),
                ),
                (
                    "event_visibility",
                    models.ManyToManyField(
                        blank=True,
                        help_text="Events with any of these visibilities will be posted to this channel. Visibility (who can see the channel) is managed in Discord directly — this only controls which events get sent here.",
                        related_name="event_channels",
                        to="opcalendar.eventvisibility",
                    ),
                ),
            ],
            options={
                "verbose_name": "Event Channel",
                "verbose_name_plural": "Event Channels",
            },
        ),
    ]



# Generated by Django 4.2.11 on 2024-05-29 12:54

from django.conf import settings
from django.db import migrations, models

import opcalendar.models


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("opcalendar", "0026_alter_event_repeat_event"),
    ]

    operations = [
        migrations.AddField(
            model_name="eventmember",
            name="comment",
            field=models.CharField(
                blank=True, help_text="Optional comment about the event", max_length=100
            ),
        ),
        migrations.AddField(
            model_name="eventmember",
            name="status",
            field=models.CharField(
                choices=[("A", "Attending"), ("M", "Maybe"), ("D", "Declined")],
                default="A",
                max_length=1,
            ),
        ),
        migrations.CreateModel(
            name="UserSettings",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "disable_discord_notifications",
                    models.BooleanField(
                        default=True, verbose_name="Discord Notifications"
                    ),
                ),
                (
                    "use_local_times",
                    models.BooleanField(default=True, verbose_name="Use Local Times"),
                ),
                (
                    "user",
                    models.ForeignKey(
                        blank=True,
                        default=None,
                        null=True,
                        on_delete=models.SET(opcalendar.models.get_sentinel_user),
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "verbose_name": "User Settings",
                "verbose_name_plural": "User Settings",
                "default_permissions": (),
            },
        ),
    ]



import os

from celery import Celery
from celery.app import trace

# set the default Django settings module for the 'celery' program.
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "testauth.settings.local")

from django.conf import settings  # noqa

app = Celery("testauth")

# Using a string here means the worker don't have to serialize
# the configuration object to child processes.
app.config_from_object("django.conf:settings")

# Automatically try to establish the connection to the AMQP broker on
# Celery startup if it is unavailable.
app.conf.broker_connection_retry_on_startup = True


# setup priorities ( 0 Highest, 9 Lowest )
app.conf.broker_transport_options = {
    "priority_steps": list(range(10)),  # setup que to have 10 steps
    "queue_order_strategy": "priority",  # setup que to use prio sorting
}
app.conf.task_default_priority = (
    5  # anything called with the task.delay() will be given normal priority (5)
)
app.conf.worker_prefetch_multiplier = (
    1  # only prefetch single tasks at a time on the workers so that prio tasks happen
)

app.conf.ONCE = {"backend": "allianceauth.services.tasks.DjangoBackend", "settings": {}}

# Load task modules from all registered Django app configs.
app.autodiscover_tasks(lambda: settings.INSTALLED_APPS)

# Remove result from default log message on task success
trace.LOG_SUCCESS = "Task %(name)s[%(id)s] succeeded in %(runtime)ss"



from datetime import timedelta

from allianceauth.services.hooks import get_extension_logger
from django import forms
from django.forms import ModelForm
from django.forms.widgets import TextInput

from opcalendar.models import (
    Event,
    EventCategory,
    EventHost,
    EventMember,
    EventVisibility,
    UserSettings,
)

logger = get_extension_logger(__name__)


class EventForm(ModelForm):
    end_time_mode = forms.ChoiceField(
        choices=[("duration", "Duration"), ("custom", "Set end time")],
        widget=forms.RadioSelect,
        initial="duration",
        required=False,
        label="End time",
    )
    duration = forms.IntegerField(
        required=False,
        min_value=1,
        label="Duration (hours)",
        widget=forms.NumberInput(attrs={"placeholder": "e.g. 2"}),
    )
    enable_repeat = forms.BooleanField(
        required=False,
        label="Repeat event",
    )
    notification_timings = forms.MultipleChoiceField(
        choices=Event.NOTIFICATION_TIMING_CHOICES,
        widget=forms.CheckboxSelectMultiple,
        required=False,
        initial=[Event.NOTIFY_NOW],
        label="Discord ping timing",
        help_text=(
            "When should this event be pinged on Discord? "
            "Select 'Custom date and time' to also set a specific date below."
        ),
    )

    field_order = [
        "operation_type",
        "title",
        "host",
        "event_visibility",
        "doctrine",
        "formup_system",
        "fc",
        "description",
        "start_time",
        "end_time_mode",
        "duration",
        "end_time",
        "important",
        "enable_repeat",
        "repeat_event",
        "repeat_interval",
        "repeat_times",
        "notification_timings",
        "notification_custom_date",
    ]

    class Meta:
        model = Event
        # datetime-local is a HTML5 input type, format to make date time show on fields
        exclude = [
            "user",
            "eve_character",
            "created_date",
            "external",
            "notifications_sent",
        ]

    def __init__(self, *args, **kwargs):

        super(EventForm, self).__init__(*args, **kwargs)

        self.fields["start_time"].input_formats = ("%Y-%m-%dT%H:%M",)
        self.fields["end_time"].input_formats = ("%Y-%m-%dT%H:%M",)
        self.fields["end_time"].required = False
        self.fields["repeat_event"].required = False
        self.fields["repeat_times"].required = False
        self.fields["notification_custom_date"].input_formats = ("%Y-%m-%dT%H:%M",)
        self.fields["host"].queryset = EventHost.objects.filter(external=False)
        self.fields["event_visibility"].required = True
        self.fields["event_visibility"].queryset = EventVisibility.objects.filter(
            is_visible=True, is_active=True
        )
        try:
            self.initial["event_visibility"] = EventVisibility.objects.get(
                is_default=True
            )
        except Exception:
            logger.debug("Form defaults: No default visibility set")

        try:
            self.initial["host"] = EventHost.objects.get(is_default=True)
        except Exception:
            logger.debug("Form defaults: No default host set")

    def clean(self):
        cleaned_data = super().clean()
        end_time_mode = cleaned_data.get("end_time_mode")
        start_time = cleaned_data.get("start_time")

        if end_time_mode == "duration":
            duration = cleaned_data.get("duration")
            if not duration:
                self.add_error("duration", "Please select a duration.")
            elif start_time:
                cleaned_data["end_time"] = start_time + timedelta(hours=int(duration))
        else:
            if not cleaned_data.get("end_time"):
                self.add_error("end_time", "End time is required.")

        if not cleaned_data.get("enable_repeat"):
            cleaned_data["repeat_event"] = ""
            cleaned_data["repeat_times"] = 0

        # Clear notification_custom_date if custom timing not selected
        timings = cleaned_data.get("notification_timings") or []
        if Event.NOTIFY_CUSTOM not in timings:
            cleaned_data["notification_custom_date"] = None
            if "notification_custom_date" in self.errors:
                del self.errors["notification_custom_date"]

        return cleaned_data


class EventEditForm(ModelForm):
    end_time_mode = forms.ChoiceField(
        choices=[("duration", "Duration"), ("custom", "Set end time")],
        widget=forms.RadioSelect,
        initial="custom",
        required=False,
        label="End time",
    )
    duration = forms.IntegerField(
        required=False,
        min_value=1,
        label="Duration (hours)",
        widget=forms.NumberInput(attrs={"placeholder": "e.g. 2"}),
    )
    enable_repeat = forms.BooleanField(
        required=False,
        label="Repeat event",
    )
    notification_timings = forms.MultipleChoiceField(
        choices=Event.NOTIFICATION_TIMING_CHOICES,
        widget=forms.CheckboxSelectMultiple,
        required=False,
        label="Discord ping timing",
        help_text=(
            "When should this event be pinged on Discord? "
            "Select 'Custom date and time' to also set a specific date below."
        ),
    )

    field_order = [
        "operation_type",
        "title",
        "host",
        "event_visibility",
        "doctrine",
        "formup_system",
        "fc",
        "description",
        "start_time",
        "end_time_mode",
        "duration",
        "end_time",
        "important",
        "enable_repeat",
        "repeat_event",
        "repeat_interval",
        "repeat_times",
        "notification_timings",
        "notification_custom_date",
    ]

    class Meta:
        model = Event
        exclude = [
            "user",
            "eve_character",
            "created_date",
            "external",
            "notifications_sent",
        ]

    def __init__(self, *args, **kwargs):
        super(EventEditForm, self).__init__(*args, **kwargs)

        self.fields["start_time"].input_formats = ("%Y-%m-%dT%H:%M",)
        self.fields["end_time"].input_formats = ("%Y-%m-%dT%H:%M",)
        self.fields["end_time"].required = False
        self.fields["repeat_event"].required = False
        self.fields["repeat_interval"].required = False
        self.fields["repeat_times"].required = False
        self.fields["notification_custom_date"].input_formats = ("%Y-%m-%dT%H:%M",)
        self.fields["host"].queryset = EventHost.objects.filter(external=False)
        self.fields["event_visibility"].required = True
        self.fields["event_visibility"].queryset = EventVisibility.objects.filter(
            is_visible=True, is_active=True
        )

        # Pre-populate non-model fields from the existing instance
        if self.instance and self.instance.pk:
            if self.instance.repeat_event:
                self.initial["enable_repeat"] = True
            self.initial["end_time_mode"] = "custom"

    def clean(self):
        cleaned_data = super().clean()

        # End time: duration or custom
        end_time_mode = cleaned_data.get("end_time_mode")
        start_time = cleaned_data.get("start_time")

        if end_time_mode == "duration":
            duration = cleaned_data.get("duration")
            if not duration:
                self.add_error("duration", "Please select a duration.")
            elif start_time:
                cleaned_data["end_time"] = start_time + timedelta(hours=int(duration))
        else:
            if not cleaned_data.get("end_time"):
                self.add_error("end_time", "End time is required.")

        # Repeat
        if not cleaned_data.get("enable_repeat"):
            cleaned_data["repeat_event"] = ""
            cleaned_data["repeat_times"] = 0

        # Notification custom date
        timings = cleaned_data.get("notification_timings") or []
        if (
            Event.NOTIFY_CUSTOM not in timings
            and "notification_custom_date" in self.errors
        ):
            del self.errors["notification_custom_date"]
            cleaned_data["notification_custom_date"] = None

        return cleaned_data


class SignupForm(forms.Form):
    username = forms.CharField(
        widget=forms.TextInput(
            attrs={"class": "form-control", "placeholder": "Username"}
        )
    )
    password = forms.CharField(
        widget=forms.PasswordInput(
            attrs={"class": "form-control", "placeholder": "Password"}
        )
    )


class AddMemberForm(forms.ModelForm):
    class Meta:
        model = EventMember
        fields = ["character", "status", "comment"]
        widgets = {
            "comment": forms.TextInput(
                attrs={
                    "class": "form-control",
                    "maxlength": 100,
                    "placeholder": "Optional comment",
                }
            ),
        }


class AddCategoryForm(forms.ModelForm):
    class Meta:
        model = EventCategory
        fields = "__all__"


class EventVisibilityAdminForm(forms.ModelForm):
    class Meta:
        model = EventVisibility
        fields = "__all__"
        widgets = {
            "color": TextInput(attrs={"type": "color"}),
        }


class EventCategoryAdminForm(forms.ModelForm):
    class Meta:
        model = EventCategory
        fields = "__all__"
        widgets = {
            "color": TextInput(attrs={"type": "color"}),
        }


class UserSettingsForm(forms.ModelForm):
    class Meta:
        model = UserSettings
        fields = ["disable_discord_notifications", "use_local_times"]
        labels = {
            "disable_discord_notifications": "Disable all direct discord notifications",
            "use_local_times": "Show all events in local time instead of EVE time",
        }



# Generated by Django 3.1.2 on 2020-11-04 11:27

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("opcalendar", "0004_event_host"),
    ]

    operations = [
        migrations.AlterField(
            model_name="event",
            name="host",
            field=models.ForeignKey(
                default=1,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="opcalendar.eventhost",
            ),
        ),
    ]



import logging
import operator
from itertools import chain

from allianceauth.services.modules.discord.models import DiscordUser
from asgiref.sync import sync_to_async
from discord import Embed
from discord.colour import Color
from discord.commands import Option
from discord.ext import commands, tasks
from django.conf import settings
from django.db.models import F
from django.utils import timezone

from opcalendar.app_settings import get_site_url
from opcalendar.models import Event, EventChannel, EventMember, IngameEvents

logger = logging.getLogger(__name__)

SIGNUP_STATUS_MAP = {
    "attending": EventMember.Status.ATTENDING,
    "maybe": EventMember.Status.MAYBE,
    "declined": EventMember.Status.DECLINED,
}

SIGNUP_NUMBER_OPTION_DESCRIPTION = "Event number from the pinned summary list"
SIGNUP_STATUS_OPTION_DESCRIPTION = "Sign-up status"
SIGNUP_STATUS_CHOICES = list(SIGNUP_STATUS_MAP)
SIGNUP_STATUS_DEFAULT = "attending"


def _upcoming_event_qs(event_channel):
    return Event.objects.select_related("operation_type").filter(
        event_visibility__in=event_channel.event_visibility.all(),
        start_time__gte=timezone.now(),
    )


def _upcoming_ingame_event_qs(event_channel):
    return IngameEvents.objects.annotate(start_time=F("event_start_date")).filter(
        owner__event_visibility__in=event_channel.event_visibility.all(),
        start_time__gte=timezone.now(),
    )


def _upcoming_events(event_channel):
    """Returns Event + IngameEvents combined, sorted by start time."""
    events = chain(
        _upcoming_event_qs(event_channel), _upcoming_ingame_event_qs(event_channel)
    )
    return sorted(events, key=operator.attrgetter("start_time"))


MAX_SUMMARY_EVENTS = 20


def _build_summary_embed(event_channel, events):
    embed = Embed(title="Upcoming events")
    embed.colour = Color.green()
    if not events:
        embed.description = "No upcoming events scheduled."
        return embed

    shown_events = events[:MAX_SUMMARY_EVENTS]
    embed.description = (
        "Sign up with `/signup`or directlyt at [AUTH]({}/opcalendar)".format(
            get_site_url()
        )
    )

    numbered_events = list(enumerate(shown_events, start=1))
    for index, event in reversed(numbered_events):
        start_unix = int(event.start_time.timestamp())
        if isinstance(event, Event):
            value = (
                "**Type:** {} {}\n**Formup:** {}\n**FC:** {}\n"
                "**Time:** <t:{}:F> (<t:{}:R>)"
            ).format(
                event.operation_type.name,
                event.operation_type.ticker,
                event.formup_system,
                event.fc,
                start_unix,
                start_unix,
            )
        else:
            value = (
                "**Ingame Event** hosted by {}\n**Time:** <t:{}:F> (<t:{}:R>)".format(
                    event.owner_name,
                    start_unix,
                    start_unix,
                )
            )

        embed.add_field(
            name="{}. {}".format(index, event.title),
            value=value,
            inline=False,
        )

    shown_count = len(shown_events)
    total_count = len(events)
    if total_count > shown_count:
        count_text = "Showing {} of {} upcoming events.".format(
            shown_count, total_count
        )
    else:
        count_text = "Showing {} upcoming event{}.".format(
            shown_count, "" if shown_count == 1 else "s"
        )

    embed.set_footer(text="{} Check out all ops on the opcalendar".format(count_text))

    return embed


@sync_to_async
def _get_enabled_event_channels():
    return list(EventChannel.objects.filter(enabled=True))


@sync_to_async
def _get_upcoming_events(event_channel):
    return _upcoming_events(event_channel)


@sync_to_async
def _render_summary_embed(event_channel, events):
    return _build_summary_embed(event_channel, events)


@sync_to_async
def _save_summary_message_id(event_channel, message_id):
    event_channel.summary_message_id = message_id
    event_channel.save(update_fields=["summary_message_id"])


@sync_to_async
def _get_event_channel_for_signup(channel_id):
    return EventChannel.objects.filter(channel_id=channel_id, enabled=True).first()


@sync_to_async
def _get_discord_user(uid):
    return (
        DiscordUser.objects.filter(uid=uid)
        .select_related("user__profile__main_character")
        .exclude(user__profile__main_character__isnull=True)
        .first()
    )


@sync_to_async
def _record_signup(event, character, status):
    EventMember.objects.update_or_create(
        event=event, character=character, defaults={"status": status}
    )


class EventChannelSync(commands.Cog):
    def __init__(self, bot):
        self.bot = bot
        self.sync_summaries.start()

    def cog_unload(self):
        self.sync_summaries.cancel()

    @tasks.loop(seconds=60)
    async def sync_summaries(self):
        for event_channel in await _get_enabled_event_channels():
            channel = self.bot.get_channel(event_channel.channel_id)
            if channel is None:
                logger.error(
                    "Could not find Discord channel %s for event channel %s",
                    event_channel.channel_id,
                    event_channel,
                )
                continue
            try:
                await self._sync_summary(channel, event_channel)
            except Exception:
                logger.exception("Failed to sync summary for %s", event_channel)

    @sync_summaries.before_loop
    async def before_sync_summaries(self):
        await self.bot.wait_until_ready()

    async def _sync_summary(self, channel, event_channel):
        events = await _get_upcoming_events(event_channel)
        embed = await _render_summary_embed(event_channel, events)

        if event_channel.summary_message_id:
            try:
                message = await channel.fetch_message(event_channel.summary_message_id)
                await message.edit(embed=embed)
                return
            except Exception:
                logger.warning(
                    "Pinned summary message %s for channel %s no longer exists, reposting",
                    event_channel.summary_message_id,
                    event_channel,
                )

        message = await channel.send(embed=embed)
        try:
            await message.pin()
        except Exception:
            logger.warning(
                "Could not pin summary message for channel %s", event_channel
            )
        await _save_summary_message_id(event_channel, message.id)

    @commands.slash_command(name="signup", guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def signup(
        self,
        ctx,
        number: Option(int, SIGNUP_NUMBER_OPTION_DESCRIPTION, required=True),
        status: Option(
            str,
            SIGNUP_STATUS_OPTION_DESCRIPTION,
            choices=SIGNUP_STATUS_CHOICES,
            required=False,
            default=SIGNUP_STATUS_DEFAULT,
        ),
    ):
        """Sign up for an upcoming event by its number in the pinned summary"""
        event_channel = await _get_event_channel_for_signup(ctx.channel.id)
        if event_channel is None:
            await ctx.respond(
                "This channel isn't set up for event sign-ups.", ephemeral=True
            )
            return

        status_key = status.lower()
        member_status = SIGNUP_STATUS_MAP.get(status_key)
        if member_status is None:
            await ctx.respond(
                "Unknown status `{}`. Use one of: {}".format(
                    status, ", ".join(SIGNUP_STATUS_MAP)
                ),
                ephemeral=True,
            )
            return

        events = await _get_upcoming_events(event_channel)
        if number < 1 or number > len(events):
            await ctx.respond(
                "No event numbered `{}` in the current summary. Check the pinned list.".format(
                    number
                ),
                ephemeral=True,
            )
            return

        event = events[number - 1]
        if not isinstance(event, Event):
            await ctx.respond(
                "Sign-ups aren't available for in-game calendar events "
                "(`{}`) - this entry is shown for reference only.".format(event.title),
                ephemeral=True,
            )
            return

        discord_user = await _get_discord_user(ctx.author.id)
        if discord_user is None:
            await ctx.respond(
                "Your Discord account isn't linked to an AllianceAuth character with a main character set.",
                ephemeral=True,
            )
            return

        character = discord_user.user.profile.main_character
        await _record_signup(event, character, member_status)
        await ctx.respond(
            "Marked **{}** as **{}** for **{}**.".format(
                character, status_key, event.title
            )
        )


def setup(bot):
    bot.add_cog(EventChannelSync(bot))



from allianceauth import hooks
from allianceauth.services.hooks import MenuItemHook, UrlHook
from django.utils.translation import gettext_lazy as _
from .models import Event
from django.db.models import Q
from django.utils import timezone
from .app_settings import OPCALENDAR_SHOW_EVENT_COUNTER

from . import urls


class OpcalendarMenuItem(MenuItemHook):
    """This class ensures only authorized users will see the menu entry"""

    def __init__(self):
        # setup menu entry for sidebar
        MenuItemHook.__init__(
            self,
            _("Operation Calendar"),
            "far fa-calendar-alt",
            "opcalendar:calendar",
            navactive=["opcalendar:calendar"],
        )

    def render(self, request):
        if request.user.has_perm("opcalendar.basic_access"):
            if OPCALENDAR_SHOW_EVENT_COUNTER:
                user_main_character = request.user.profile.main_character
                now = timezone.now()
                self.count = (
                    Event.objects.filter(
                        Q(
                            event_visibility__restricted_to_group__in=request.user.groups.all()
                        )
                        | Q(event_visibility__restricted_to_group__isnull=True),
                    )
                    .filter(
                        Q(
                            event_visibility__restricted_to_state=request.user.profile.state
                        )
                        | Q(event_visibility__restricted_to_state__isnull=True),
                    )
                    .filter(
                        Q(eventmember__isnull=True)
                        | ~Q(eventmember__character=user_main_character),
                        external=False,
                        start_time__gte=now,
                    )
                    .distinct()
                    .count()
                )
            return MenuItemHook.render(self, request)
        return ""


@hooks.register("menu_item_hook")
def register_menu():
    return OpcalendarMenuItem()


@hooks.register("url_hook")
def register_urls():
    return UrlHook(
        urls,
        "opcalendar",
        r"^opcalendar/",
        excluded_views=["opcalendar.views.EventFeed"],
    )


@hooks.register("discord_cogs_hook")
def register_cogs():
    return ["opcalendar.cogs.ops"]



from allianceauth.services.hooks import get_extension_logger
from django.db.models.signals import post_save, pre_delete
from django.dispatch import receiver
from django.utils import timezone
from .app_settings import (
    OPCALENDAR_NOTIFY_IMPORTS,
    OPCALENDAR_NOTIFY_REPEAT_EVENTS,
    get_site_url,
)
from .models import Event, IngameEvents, _get
from .providers import esi

logger = get_extension_logger(__name__)

RED = 16711710
BLUE = 42751
GREEN = 6684416


def send_webhook(embed, hook, eve_time, content=None):
    """Helper function to send the webhook."""
    old = timezone.now() > eve_time
    if hook and hook.webhook and hook.webhook.enabled:
        if old and hook.ignore_past_fleets:
            logger.debug("Event is in the past, not sending webhook.")
        else:
            hook.webhook.send_embed(embed, content=content)


@receiver(post_save, sender=Event)
@receiver(post_save, sender=IngameEvents)
def fleet_saved(sender, instance, created, **kwargs):
    logger.debug("A new operation has been scheduled: %s", instance)
    col = GREEN if created else BLUE

    if sender == IngameEvents and OPCALENDAR_NOTIFY_IMPORTS:
        handle_ingame_event(instance, created, col)
    elif sender == Event:
        handle_standard_event(instance, created, col)


def handle_ingame_event(instance, created, col):
    """Handle IngameEvents notifications."""
    try:
        logger.debug("New signal created for Ingame Calendar Event: %s", instance.title)
        url = get_site_url() + f"/opcalendar/ingame/event/{instance.pk}/details/"
        message = f"New ingame calendar event: {instance.title}"
        category_map = {
            "alliance": "alliances",
            "corporation": "corporations",
            "character": "characters",
        }
        category = category_map.get(instance.owner_type, "characters")

        result = esi.client.Universe.PostUniverseIds(
            body=[instance.owner_name]
        ).result()
        matches = _get(result, category) or []
        if not matches:
            logger.debug(
                "No ESI match found for %s '%s', skipping notification",
                instance.owner_type,
                instance.owner_name,
            )
            return

        entity_id = _get(matches[0], "id")

        logger.debug("Entity data is %s", entity_id)
        portrait, ticker = get_entity_details(entity_id, instance.owner_type)

        embed = {
            "title": message,
            "description": instance.text,
            "url": url,
            "color": col,
            "fields": [
                {"name": "Owner", "value": instance.owner_name, "inline": True},
                {
                    "name": "Eve Time",
                    "value": instance.event_start_date.strftime("%Y-%m-%d %H:%M:%S"),
                },
            ],
            "footer": {"icon_url": portrait, "text": f"{instance.owner_name} {ticker}"},
        }
        send_webhook(embed, instance.owner.event_visibility, instance.event_start_date)
    except Exception as e:
        logger.exception(e)


def handle_standard_event(instance, created, col):
    """Handle standard Event notifications.

    The instant Discord ping only fires when "now" is one of the selected
    notification timings (and hasn't been sent yet). Other selected timings
    are picked up later by the send_scheduled_event_notifications task.
    """
    if not _should_send_now(instance):
        return

    if instance.repeat_event and not OPCALENDAR_NOTIFY_REPEAT_EVENTS:
        return

    handle_event_notification(instance, created, col, "New event:")
    _mark_notification_sent(instance, Event.NOTIFY_NOW)


def _should_send_now(instance):
    """Whether the immediate ('now') Discord ping should be sent for this event."""
    return (
        Event.NOTIFY_NOW in instance.notification_timings
        and Event.NOTIFY_NOW not in instance.notifications_sent
    )


def _mark_notification_sent(instance, timing):
    """Records a notification timing as sent without re-triggering save signals."""
    if timing in instance.notifications_sent:
        return
    instance.notifications_sent.append(timing)
    Event.objects.filter(pk=instance.pk).update(
        notifications_sent=instance.notifications_sent
    )


def handle_event_notification(instance, created, col, message_prefix, content=None):
    """Generate and send event notifications."""
    try:
        logger.debug("New signal fleet created for event %s", instance.title)
        url = get_site_url() + f"/opcalendar/event/{instance.pk}/details/"
        message = f"{message_prefix} {instance.title}"

        main_char = instance.eve_character
        if main_char:
            portrait = main_char.portrait_url_64
            character_name = main_char.character_name
            ticker = f"[{main_char.corporation_ticker}]"
        else:
            portrait = character_name = ticker = ""

        embed = {
            "title": message,
            "description": instance.description,
            "url": url,
            "color": col,
            "fields": [
                {"name": "FC", "value": instance.fc, "inline": True},
                {"name": "Type", "value": instance.operation_type.name, "inline": True},
                {"name": "Formup", "value": instance.formup_system, "inline": True},
                {
                    "name": "Eve Time",
                    "value": instance.start_time.strftime("%Y-%m-%d %H:%M:%S"),
                },
            ],
            "footer": {
                "icon_url": portrait,
                "text": f"{character_name} {ticker}, {instance.host}",
            },
        }
        send_webhook(
            embed, instance.event_visibility, instance.start_time, content=content
        )
    except Exception as e:
        logger.exception(e)


SCHEDULED_NOTIFICATION_PREFIXES = {
    Event.NOTIFY_1H_BEFORE: "Reminder: Fleet starting in 1 hour:",
    Event.NOTIFY_1D_BEFORE: "Reminder: Fleet starting in 1 day:",
    Event.NOTIFY_1W_BEFORE: "Reminder: Fleet starting in 1 week:",
    Event.NOTIFY_CUSTOM: "Reminder:",
}


def send_scheduled_notification(instance, timing):
    """Sends a scheduled Discord ping for the given notification timing and
    records it as sent. Called from send_scheduled_event_notifications."""
    message_prefix = SCHEDULED_NOTIFICATION_PREFIXES.get(timing, "Reminder:")
    handle_event_notification(instance, False, BLUE, message_prefix)
    _mark_notification_sent(instance, timing)


PING_TARGET_MENTIONS = {
    "here": "@here",
    "everyone": "@everyone",
}


def send_starting_now_ping(instance, ping_target):
    """Sends an immediate 'starting now' Discord ping with an @here/@everyone
    mention. Triggered manually by the event owner from the event details page."""
    mention = PING_TARGET_MENTIONS.get(ping_target)
    handle_event_notification(instance, False, GREEN, "Starting now:", content=mention)


def get_entity_details(entity_id, owner_type):
    """Retrieve the entity details based on the owner type."""
    if owner_type == "alliance":
        portrait = f"https://images.evetech.net/alliances/{entity_id}/logo"
        alliance = esi.client.Alliance.GetAlliancesAllianceId(
            alliance_id=entity_id
        ).result()
        ticker = f"[{_get(alliance, 'ticker')}]"
    elif owner_type == "corporation":
        portrait = f"https://images.evetech.net/corporations/{entity_id}/logo"
        corporation = esi.client.Corporation.GetCorporationsCorporationId(
            corporation_id=entity_id
        ).result()
        ticker = f"[{_get(corporation, 'ticker')}]"
    else:  # character
        portrait = f"https://images.evetech.net/characters/{entity_id}/portrait"
        ticker = ""
    return portrait, ticker


@receiver(pre_delete, sender=Event)
@receiver(pre_delete, sender=IngameEvents)
def fleet_deleted(sender, instance, **kwargs):
    logger.debug("An operation has been deleted: %s", instance)

    if sender == IngameEvents and OPCALENDAR_NOTIFY_IMPORTS:
        handle_event_deletion(instance, "Deleted Ingame Calendar Event:", RED)
    elif sender == Event:
        handle_event_deletion(instance, "Event deleted:", RED)


def handle_event_deletion(instance, message_prefix, color):
    """Handle event deletions and send notifications."""
    try:
        url = get_site_url() + f"/opcalendar/event/{instance.pk}/details/"
        message = f"{message_prefix} {instance.title}"

        main_char = instance.eve_character
        if main_char:
            portrait = main_char.portrait_url_64
            character_name = main_char.character_name
            ticker = f"[{main_char.corporation_ticker}]"
        else:
            portrait = character_name = ticker = ""

        embed = {
            "title": message,
            "description": instance.description,
            "url": url,
            "color": color,
            "fields": [
                {"name": "FC", "value": instance.fc, "inline": True},
                {"name": "Type", "value": instance.operation_type.name, "inline": True},
                {"name": "Formup", "value": instance.formup_system, "inline": True},
                {
                    "name": "Eve Time",
                    "value": instance.start_time.strftime("%Y-%m-%d %H:%M:%S"),
                },
            ],
            "footer": {
                "icon_url": portrait,
                "text": f"{character_name} {ticker}, {instance.host}",
            },
        }
        send_webhook(embed, instance.event_visibility, instance.start_time)
    except Exception as e:
        logger.exception(e)



from calendar import HTMLCalendar
from datetime import date
from itertools import chain

from allianceauth.services.hooks import get_extension_logger
from django.db.models import F, Q
from django.utils import timezone

from .app_settings import (
    OPCALENDAR_DISPLAY_MOONMINING,
    OPCALENDAR_DISPLAY_MOONMINING_TAGS,
    OPCALENDAR_DISPLAY_STRUCTURETIMERS,
    moonmining_active,
    structuretimers_active,
)
from .models import Event, EventMember, IngameEvents

if structuretimers_active():
    from structuretimers.models import Timer

if moonmining_active():
    from moonmining.models import Extraction

logger = get_extension_logger(__name__)


class Calendar(HTMLCalendar):
    def __init__(self, year=None, month=None, user=None):
        self.year = year
        self.month = month
        self.user = user
        super(Calendar, self).__init__()

    def formatday(
        self,
        day,
        events,
        ingame_events,
        structuretimer_events,
        moonmining_events,
        member_statuses,
    ):
        structuretimers_per_day = []
        moonmining_per_day = []
        events_per_day = events.filter(start_time__day=day)
        ingame_events_per_day = ingame_events.filter(event_start_date__day=day)

        if structuretimers_active() and OPCALENDAR_DISPLAY_STRUCTURETIMERS:
            structuretimers_per_day = structuretimer_events.filter(start_time__day=day)

        if moonmining_active() and OPCALENDAR_DISPLAY_MOONMINING:
            moonmining_per_day = moonmining_events.filter(chunk_arrival_at__day=day)

        all_events_per_day = sorted(
            chain(
                ((event, f"event-{event.id}") for event in events_per_day),
                (
                    (event, f"ingame-{event.event_id}")
                    for event in ingame_events_per_day
                ),
                ((event, f"struct-{event.id}") for event in structuretimers_per_day),
                ((event, f"moon-{event.id}") for event in moonmining_per_day),
            ),
            key=lambda item: (
                item[0].start_time
                if hasattr(item[0], "start_time")
                else item[0].chunk_arrival_at
            ),
        )

        event_html_list = []
        standardized_events_per_day = []
        if day != 0:
            for event, unique_id in all_events_per_day:
                standardized_event = {
                    "id": unique_id,
                    "start_time": (
                        event.start_time
                        if hasattr(event, "start_time")
                        else event.chunk_arrival_at
                    ),
                    "end_time": getattr(event, "end_time", None),
                    "title": getattr(event, "title", "Unnamed Event"),
                    "description": getattr(event, "description", ""),
                    "type": type(event).__name__,
                }
                standardized_events_per_day.append(standardized_event)

                if type(event).__name__ == "Timer":
                    # Objective icon (who it belongs to)
                    objective_icon_map = {
                        "HO": '<i class="fas fa-crosshairs" style="color:#e74c3c;" title="Hostile"></i>',
                        "FR": '<i class="fas fa-handshake" style="color:#27ae60;" title="Friendly"></i>',
                        "NE": '<i class="fas fa-circle" style="color:#95a5a6;font-size:.6em;vertical-align:middle;" title="Neutral"></i>',
                        "UN": '<i class="fas fa-question" style="color:#95a5a6;" title="Undefined"></i>',
                    }

                    # Timer type icon and label
                    type_icon_map = {
                        "AR": '<i class="fas fa-shield-alt" style="color:#e67e22;" title="Armor"></i>',
                        "HL": '<i class="fas fa-cube" style="color:#7f8c8d;" title="Hull"></i>',
                        "FI": '<i class="fas fa-bomb" style="color:#e74c3c;" title="Final"></i>',
                        "AN": '<i class="fas fa-anchor" style="color:#27ae60;" title="Anchoring"></i>',
                        "UA": '<i class="fas fa-anchor" style="color:#e74c3c;" title="Unanchoring"></i>',
                        "MM": '<i class="fas fa-moon" style="color:#9b59b6;" title="Moon Mining"></i>',
                        "NO": "",
                    }
                    type_label_map = {
                        "AR": "Armor",
                        "HL": "Hull",
                        "FI": "Final",
                        "AN": "Anchoring",
                        "UA": "Unanchoring",
                        "MM": "Moon Mining",
                        "NO": "Preliminary",
                    }

                    objective_icon = objective_icon_map.get(event.objective, "")
                    type_icon = type_icon_map.get(event.timer_type, "")
                    type_label = type_label_map.get(event.timer_type, "Structure")
                    important_class = (
                        "structuretimer-important" if event.is_important else ""
                    )

                    # Generate the HTML for the Timer event
                    event_html_list.append(
                        f'<div class="event {"past-event" if timezone.now() > event.date else "future-event"} event-structuretimer {important_class}">'
                        f'<i class="event-type-icon fas fa-hourglass-half" style="color: #e08e3e;"></i>'
                        f'<div class="event-row">'
                        f'<span class="event-time" id="event-time-{unique_id}">{event.date.strftime("%H:%M")}</span>'
                        f'<span class="event-title">{event.eve_solar_system.name} - {event.structure_type.name}</span>'
                        f"</div>"
                        f'<span class="event-owner">{objective_icon} {type_icon} {type_label} timer</span>'
                        f"</div>"
                    )

                elif type(event).__name__ == "Extraction":
                    if self.user.has_perm("moonmining.extractions_access"):
                        # Extract relevant details for the extraction event
                        refinery = event.refinery.name
                        system = (
                            event.refinery.moon.eve_moon.eve_planet.eve_solar_system.name
                        )
                        structure = refinery.replace(system, "")

                        # Generate the display name with or without moon tags
                        display_name = (
                            (
                                event.refinery.moon.rarity_tag_html
                                + '<span class="event-moon-name">'
                                + structure[3:]
                                + "</span>"
                            )
                            if OPCALENDAR_DISPLAY_MOONMINING_TAGS
                            else "<span>" + structure[3:] + "</span>"
                        )

                        # Generate the HTML for the Extraction event
                        event_html_list.append(
                            f'<a class="nostyling" href="/moonmining/extraction/{event.id}?new_page=yes">'
                            f'<div class="event {"past-event" if timezone.now() > event.chunk_arrival_at else "future-event"} event-moonmining">'
                            f'<i class="event-type-icon fas fa-moon" style="color: #3f9fd1;"></i>'
                            f'<div class="event-row">'
                            f'<span class="event-time" id="event-time-{unique_id}">{event.chunk_arrival_at.strftime("%H:%M")}</span>'
                            f'<span class="event-title">{event.refinery.moon.eve_moon.name}</span>'
                            f"</div>"
                            f'<div class="event-moon-details">'
                            f"{display_name}"
                            f"</div>"
                            f"</div>"
                            f"</a>"
                        )

                elif type(event).__name__ in ["Event", "IngameEvents"]:
                    # Determine start time, title, and owner for the event
                    start_time = (
                        event.start_time.strftime("%H:%M")
                        if hasattr(event, "start_time")
                        else event.event_start_date.strftime("%H:%M")
                    )
                    title = (
                        f"{event.operation_type.ticker} {event.title}"
                        if hasattr(event, "operation_type")
                        else event.title
                    )
                    owner = (
                        event.host.community
                        if hasattr(event, "host")
                        else event.owner_name
                    )

                    # Show an icon if the current user has signed up / reacted to this event
                    signup_icon = ""
                    if type(event).__name__ == "Event":
                        signup_status = member_statuses.get(event.id)
                        if signup_status == EventMember.Status.ATTENDING:
                            signup_icon = '<i class="fas fa-check-circle event-signup-icon signup-attending" title="Attending"></i>'
                        elif signup_status == EventMember.Status.MAYBE:
                            signup_icon = '<i class="fas fa-question-circle event-signup-icon signup-maybe" title="Maybe"></i>'
                        elif signup_status == EventMember.Status.DECLINED:
                            signup_icon = '<i class="fas fa-times-circle event-signup-icon signup-declined" title="Declined"></i>'

                    # Mark important events so they get the blinking indicator
                    important_class = (
                        "important-event" if getattr(event, "important", False) else ""
                    )

                    # Generate the HTML for the Event or IngameEvents
                    event_html_list.append(
                        f"<style>{event.get_event_styling}</style>"
                        f'<a class="nostyling" href="{event.get_html_url}">'
                        f'<div class="event {event.get_date_status} {event.get_visibility_class} {event.get_category_class} {event.external_tag} {important_class} {event.get_category_gutter_class}">'
                        f'<span class="important-dot"></span>'
                        f"{event.get_category_icon_html}"
                        f'<div class="event-row">'
                        f'<span class="event-time" id="event-time-{unique_id}">{start_time}</span>'
                        f'<span class="event-title">{title}</span>'
                        f"</div>"
                        f'<span class="event-owner">{owner}</span>'
                        f"{signup_icon}"
                        f"</div>"
                        f"</a>"
                    )

            d = "".join(event_html_list)

            scroll_arrows = (
                '<span class="day-scroll-arrow day-scroll-up"><i class="fas fa-chevron-up"></i></span>'
                '<span class="day-scroll-arrow day-scroll-down"><i class="fas fa-chevron-down"></i></span>'
                '<span class="day-events-more-count"></span>'
            )

            if date.today() == date(self.year, self.month, day):
                return (
                    f"<td class='today'><div class='day-cell'><div class='date'>{day}</div><div class='day-events'>{d}</div>{scroll_arrows}</div></td>",
                    standardized_events_per_day,
                )
            return (
                f"<td><div class='day-cell'><div class='date'>{day}</div><div class='day-events'>{d}</div>{scroll_arrows}</div></td>",
                standardized_events_per_day,
            )
        return "<td><div class='day-cell'></div></td>", standardized_events_per_day

    def formatweek(
        self,
        theweek,
        events,
        ingame_events,
        structuretimer_events,
        moonmining_events,
        member_statuses,
    ):
        week = ""
        all_events = []
        for d, weekday in theweek:
            day_html, day_events = self.formatday(
                d,
                events,
                ingame_events,
                structuretimer_events,
                moonmining_events,
                member_statuses,
            )
            week += day_html
            all_events.extend(day_events)
        return f"<tr> {week} </tr>", all_events

    def formatmonth(self, withyear=True):
        events = (
            Event.objects.filter(
                start_time__year=self.year,
                start_time__month=self.month,
            )
            .filter(
                Q(event_visibility__restricted_to_group__in=self.user.groups.all())
                | Q(event_visibility__restricted_to_group__isnull=True),
            )
            .filter(
                Q(event_visibility__restricted_to_state=self.user.profile.state)
                | Q(event_visibility__restricted_to_state__isnull=True),
            )
            .distinct()
        )

        ingame_events = (
            IngameEvents.objects.filter(
                event_start_date__year=self.year, event_start_date__month=self.month
            )
            .annotate(start_time=F("event_start_date"), end_time=F("event_end_date"))
            .filter(
                Q(
                    owner__event_visibility__restricted_to_group__in=self.user.groups.all()
                )
                | Q(owner__event_visibility__restricted_to_group__isnull=True),
            )
            .filter(
                Q(owner__event_visibility__restricted_to_state=self.user.profile.state)
                | Q(owner__event_visibility__restricted_to_state__isnull=True),
            )
            .distinct()
        )

        if (
            structuretimers_active()
            and OPCALENDAR_DISPLAY_STRUCTURETIMERS
            and self.user.has_perm("structuretimers.basic_access")
        ):
            structuretimer_events = (
                Timer.objects.all()
                .visible_to_user(self.user)
                .annotate(start_time=F("date"))
                .filter(date__year=self.year, date__month=self.month)
            )
        else:
            structuretimer_events = Event.objects.none()

        if moonmining_active() and OPCALENDAR_DISPLAY_MOONMINING:
            moonmining_events = (
                Extraction.objects.all()
                .annotate(start_time=F("chunk_arrival_at"))
                .filter(
                    chunk_arrival_at__year=self.year, chunk_arrival_at__month=self.month
                )
                .exclude(status="CN")
            )
        else:
            moonmining_events = Event.objects.none()

        logger.debug(
            "Returning %s extractions, display setting is %s. List is: %s"
            % (
                moonmining_events.count(),
                OPCALENDAR_DISPLAY_MOONMINING,
                moonmining_events,
            )
        )

        logger.debug("Returning %s ingame events" % ingame_events.count())

        # Build a lookup of the current user's signup/reaction status per event,
        # so we can show a status icon on events they've responded to
        member_statuses = {}
        if self.user is not None:
            user_main_character = self.user.profile.main_character
            member_statuses = dict(
                EventMember.objects.filter(
                    event__in=events, character=user_main_character
                ).values_list("event_id", "status")
            )

        cal = '<table class="calendar">\n'
        cal += f"{self.formatmonthname(self.year, self.month, withyear=withyear)}\n"
        cal += f"{self.formatweekheader()}\n"

        all_events_per_month = []

        for week in self.monthdays2calendar(self.year, self.month):
            week_html, week_events = self.formatweek(
                week,
                events,
                ingame_events,
                structuretimer_events,
                moonmining_events,
                member_statuses,
            )
            cal += f"{week_html}\n"
            all_events_per_month.extend(week_events)

        cal += "</table>"

        return cal, all_events_per_month



from esi.openapi_clients import ESIClientProvider

from . import __version__

esi = ESIClientProvider(
    compatibility_date="2025-08-26",  # <-- replace with the same value eveuniverse/providers.py uses
    ua_appname="aa-opcalendar",
    ua_version=__version__,
    operations=[
        "GetCharactersCharacterIdCalendar",
        "GetCharactersCharacterIdCalendarEventId",
        "PostUniverseIds",
        "GetAlliancesAllianceId",
        "GetCorporationsCorporationId",
    ],
)



import logging

import discord
from discord.ext import commands
from django.core.management.base import BaseCommand, CommandError

from opcalendar.app_settings import OPCALENDAR_DISCORD_BOT_TOKEN
from opcalendar.cogs.event_channel import EventChannelSync
from opcalendar.cogs.ops import Ops

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = (
        "Runs opcalendar's standalone Discord bot, which keeps a pinned "
        "summary of upcoming events synced in Discord event channels and "
        "provides the /signup and /ops slash commands. "
        "Run this as its own long-lived process (e.g. a supervisor program)."
    )

    def handle(self, *args, **options):
        if not OPCALENDAR_DISCORD_BOT_TOKEN:
            raise CommandError(
                "OPCALENDAR_DISCORD_BOT_TOKEN is not set. Add it to your "
                "local.py before running this command."
            )

        intents = discord.Intents.default()
        intents.reactions = True

        bot = commands.Bot(command_prefix="!opcalendar ", intents=intents)

        @bot.event
        async def on_ready():
            logger.info("opcalendar event channel bot logged in as %s", bot.user)
            self.stdout.write(
                self.style.SUCCESS(f"Logged in as {bot.user} — bot is now running")
            )

        async def runner():
            bot.add_cog(EventChannelSync(bot))
            bot.add_cog(Ops(bot))
            await bot.start(OPCALENDAR_DISCORD_BOT_TOKEN)

        try:
            bot.loop.run_until_complete(runner())
        except KeyboardInterrupt:
            pass
        finally:
            if not bot.is_closed():
                bot.loop.run_until_complete(bot.close())



import random
from datetime import timedelta

from django.contrib.auth.models import User
from django.core.management.base import BaseCommand
from django.utils import timezone

from opcalendar.models import Event, EventCategory, EventHost, EventVisibility

FLEET_NAMES = [
    "Stratop: Hostile Staging Cleanup",
    "Roaming Gang - Null Sec",
    "Mining Op - Moon Goo",
    "CTA: Defend Home System",
    "Training Op - New Bros Welcome",
    "Black Ops Drop",
    "Capital Escalation Response",
    "Faction Warfare Plex Run",
    "Industry Op - Hauling Run",
    "PvP Roam - Lowsec",
]

DOCTRINES = [
    "https://example.com/doctrine/armor-cruiser",
    "https://example.com/doctrine/shield-bs",
    "Whatever you've got, T1 frigates",
    "https://example.com/doctrine/capital-escalation",
    "Logistics + DPS, see doctrine sheet",
]

SYSTEMS = [
    "Jita",
    "Amarr",
    "Rens",
    "Dodixie",
    "Hek",
    "1DQ1-A",
    "Tama",
    "Rancer",
    "EC-P8R",
    "Vlillirier",
]

FCS = [
    "Some FC",
    "Another FC",
    "Fleet Boss",
    "Capital Pilot",
    "Newbro Friendly FC",
]

DESCRIPTIONS = [
    "Form up on time, bring your A-game.",
    "Casual roam, all are welcome.",
    "Mandatory attendance for corp members.",
    "Bring scout alts if you have them.",
    "Doctrine fit only, check fitting in description.",
]

# Non-custom timings only, so we don't need to also set notification_custom_date
NOTIFICATION_TIMING_POOL = [
    Event.NOTIFY_NOW,
    Event.NOTIFY_1H_BEFORE,
    Event.NOTIFY_1D_BEFORE,
    Event.NOTIFY_1W_BEFORE,
]

# name, ticker, color, icon
CATEGORIES = [
    ("Stratop", "OP", "#e74c3c", "fas fa-rocket"),
    ("Mining", "MIN", "#27ae60", "fas fa-cubes"),
    ("CTA", "CTA", "#c0392b", "fas fa-exclamation-triangle"),
    ("Roam", "ROAM", "#2980b9", "fas fa-route"),
    ("Training", "TRN", "#8e44ad", "fas fa-graduation-cap"),
    ("Industry", "IND", "#f39c12", "fas fa-industry"),
    ("Social", "SOC", "#16a085", "fas fa-glass-cheers"),
]


class Command(BaseCommand):
    help = (
        "Creates a number of random example events on the calendar for "
        "testing/demo purposes. Creates a default host/category/visibility "
        "filter if none exist yet."
    )

    def add_arguments(self, parser):
        parser.add_argument(
            "count",
            nargs="?",
            type=int,
            default=10,
            help="Number of random events to create (default: 10)",
        )

    def handle(self, *args, **options):
        count = options["count"]

        host, _ = EventHost.objects.get_or_create(
            community="Example Host",
            defaults={"is_default": True},
        )

        categories = []
        for name, ticker, color, icon in CATEGORIES:
            category, _ = EventCategory.objects.get_or_create(
                name=name,
                defaults={
                    "ticker": ticker,
                    "color": color,
                    "icon": icon,
                },
            )
            categories.append(category)

        visibility, _ = EventVisibility.objects.get_or_create(
            name="Public",
            defaults={"is_default": True, "is_visible": True},
        )

        user = User.objects.filter(is_superuser=True).first() or User.objects.first()
        if not user:
            self.stderr.write(
                self.style.ERROR(
                    "No users found in the database - create a user first."
                )
            )
            return

        profile = getattr(user, "profile", None)
        character = getattr(profile, "main_character", None)

        now = timezone.now()
        created = 0

        for i in range(count):
            start_offset = timedelta(
                days=random.randint(-3, 21),
                hours=random.randint(0, 23),
                minutes=random.choice([0, 15, 30, 45]),
            )
            start_time = (now + start_offset).replace(second=0, microsecond=0)
            end_time = start_time + timedelta(hours=random.choice([1, 2, 3, 4]))

            notification_timings = random.sample(
                NOTIFICATION_TIMING_POOL,
                k=random.randint(1, 2),
            )

            Event.objects.create(
                operation_type=random.choice(categories),
                title=f"{random.choice(FLEET_NAMES)} #{i + 1}",
                host=host,
                doctrine=random.choice(DOCTRINES),
                formup_system=random.choice(SYSTEMS),
                description=random.choice(DESCRIPTIONS),
                start_time=start_time,
                end_time=end_time,
                fc=random.choice(FCS),
                event_visibility=visibility,
                important=random.random() < 0.15,
                notification_timings=notification_timings,
                eve_character=character,
                user=user,
            )
            created += 1

        self.stdout.write(self.style.SUCCESS(f"Created {created} random events."))
